#ifndef __VNCCLIENT_H__
#define __VNCCLIENT_H__

#include <stdlib.h>
#include <stdint.h>
#include "rfbproto.h"

// #define VNCCLIENT_RBUFIZE 0x10000
#define VNCCLIENT_RBUFIZE (0xffff)
#define VNCCLIENT_SBUFSIZE (128)

extern void SocketSetBlocking(int fd, bool block);

class VncClient
{
	public:
		typedef enum
		{
			ERROR_NONE = 0,
			ERROR_FATAL,
			ERROR_WARNING
		} ErrorType;
		
		typedef void (*ErrorHandler)(VncClient *);
		typedef void (*FbHandler)(VncClient *, const void *, uint32_t);
		
		VncClient(FbHandler fbHandler, ErrorHandler errorHandler);
		~VncClient();
		
		void connect(const char *server, unsigned long port);
		void close();
		void handshake();
		void update();
		
		const char *getRemoteName() { return remoteName; }
		
		typedef struct
		{
			uint8_t depth;
			uint16_t redMax;
			uint16_t greenMax;
			uint16_t blueMax;
			uint8_t redShift;
			uint8_t greenShift;
			uint8_t blueShift;
		} PixelFormat;
		void setPixelFormat(PixelFormat *format);
		void setEncodings();
		
		typedef struct
		{
			uint16_t x, y, w, h;
		} Rect;
		void framebufferUpdateRequest(Rect *rect, bool updateOnly);
		
		void getRemoteResolution(Rect *rect)
		{
			rect->x = 0;
			rect->y = 0;
			rect->w = remoteWidth;
			rect->h = remoteHeight;
		};
		
		ErrorType catchError(char *buffer);
		
	protected:
		
		uint32_t rfbVersion;
		
		typedef enum
		{
			MODE_NORMAL = 0,
			MODE_FB
		} Mode;
		Mode mode;
		
		FbHandler handleFb;
		uint16_t fbRectangles;
		int32_t fbDataCount;
		
		void copyRfbString(char *dst, const void *src);
		
		void allocBuffers();
		
		int receive();
		int forceReceive();
		int pollReceive();
		int send(unsigned int size);
		
		int fd;
		
		void log(const char *message, ...);
		void throwError(const char *message, ...);
		void throwError(ErrorType type, const char *message, ...);
		
		struct
		{
			ErrorType type;
			char msg[512];
		} currentError;
		ErrorHandler handleError;
		
		unsigned long sbufSize;
		char *sbuf;
		unsigned long rbufSize;
		char *rbuf;
		
		unsigned short remoteWidth;
		unsigned short remoteHeight;
		char remoteName[64];
		
};



#endif /*__VNCCLIENT_H__*/

