/*
 *  Copyright (C) 2006 Thoduv.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#ifndef __INI_H__ 
#define __INI_H__

typedef struct ini_value_
{
	char name[64];
	char value[256];
	ini_value_ *next;
} ini_value_t;

typedef struct ini_section_
{
	char name[64];
	ini_section_ *next;
	ini_value_t *val;
} ini_section_t;

typedef struct
{
	ini_section_t *sec;
} ini_file_t;

ini_file_t *ini_parse(const char *filename);	/* parses an ini file and returns a struct */
ini_file_t *ini_blank_new();	/* create a new blank ini structure */
void ini_free_file(ini_file_t *ini);	/* free data of a ini structure */

ini_section_t *ini_req_section(ini_file_t *ini, const char *secs);	/* returns pointer to struct of section named "secs" or NULL if not found */
void ini_add_section(ini_file_t *ini, const char *secs);	/* add new section named "secs" */
void ini_rem_section(ini_file_t *ini, const char *secs);	/* delete section named "secs" */
void ini_mod_section(ini_file_t *ini, const char *secs, const char *secs_new);	/* rename section named "secs" to "secs_new" */

const char *ini_req_value(ini_file_t *ini, const char *secs, const char *vals);	/* returns value of value named "vals" in "secs" */
void ini_add_value(ini_file_t *ini, const char *secs, const char *vals, const char *value);	/* add new value named "vals" equal to "value", in "secs" */
void ini_rem_value(ini_file_t *ini, const char *secs, const char *vals);	/* delete value name "vals" in "secs" */
void ini_mod_value(ini_file_t *ini, const char *secs, const char *vals, const char *vals_new, const char *value_new);	/* rename value "vals" in "secs" to "vals_new" (if NULL, do not rename) and modify its value to "value_new" (if NULL, do not modify) */

int ini_write(ini_file_t *ini, const char *filename);	/* writes ini structure a new ini file */

void ini_log(ini_file_t *ini);	/* debug: dump ini structure to stderr */

int strcmp_ncase(const char *s1, const char *s2);	/* tool: case-insensitive comparison */

#endif /*__INI_H__*/

