/*
 *  Copyright (C) 2006 Thoduv.  All Rights Reserved.
 *
 *  This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 *  USA.
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "monofont.h"

u16 monofont_spaceSize = 6;

const monofont_char *monofont_putChar(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, const monofont_char *police, u16 textColor, u16 monofontFlags, char c)
{

	u16 monoLine;
	
	int i,j;
	
	const monofont_char *tChar;
	
	/* On rcupre la structure correspondante au caractre ... */
	tChar = &police[(int)c];
	
	if(monofontFlags & MONOFONT_ONLYCOMPUTE) return tChar;
	
	/* On l'affiche lignes par lignes */
	for(i = 0; i < tChar->ySize ; i++)
	{
			
		monoLine = tChar->monoData[i];
			
		/* Et on affiche chaque pixel ... */
		for(j = 0; j < tChar->xSize; j++)
		{
			if (monofontFlags & MONOFONT_OVERWRITE)
			{
				(*fnPlot)(bmpVram, 32, x + j, y + i, ((monoLine >> j) & 1) ? textColor : 0);
			}
			else
			{
				if((monoLine >> j) & 1)
					(*fnPlot)(bmpVram, 32, x + j, y + i, textColor);
			}
		}
	
	}
	
	return tChar;
	
}

int monofont_printText(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, u16 mx, u16 my, const monofont_char *police, u16 textColor, u16 monofontFlags, const char *text)
{
	int i,j;
	
	u16 yMax;
	u16 xPos, yPos;
	
	const monofont_char *tChar;
	
	const char *p;
	
	
	const char *cflagEnd;
	char cflagBuf[4];
	
	xPos = x;
	yPos = y;
	yMax = 0;
	
	p = text;
	
	/* On parcours la chaine ... */
	while(*p != '\0')
	{
		
		switch(*p)
		{
			case ' ':
				if(monofontFlags & MONOFONT_OVERWRITE)
				{
					for(i = 0; i < yMax ; i++)
					{
						/* Et on affiche chaque pixel ... */
						for(j = 0; j < monofont_spaceSize; j++)
						{
							if(~monofontFlags & MONOFONT_ONLYCOMPUTE) (*fnPlot)(bmpVram, 32, xPos + j, yPos + i, 0);
						}
					}
				}
				xPos += monofont_spaceSize;
				break;
			case '\n':
				yPos += yMax;
				xPos = x;
				break;
			case monofont_flagChar:
				if(strncmp(p, monofont_flagColor, strlen(monofont_flagColor)) == 0)
				{
					cflagEnd = strchr(p, ';');
					cflagBuf[0]=cflagBuf[1]=cflagBuf[2]=cflagBuf[3]='\0';
					memcpy(cflagBuf, p + strlen(monofont_flagColor), (cflagEnd) - (p + strlen(monofont_flagColor)));
					textColor = atoi(cflagBuf);
					p = cflagEnd;
					break;	
				}
			default:
				if(xPos >= 256) { xPos = x; yPos += yMax; }
				tChar = monofont_putChar(bmpVram, fnPlot, xPos, yPos, police, textColor, monofontFlags, *p);
				xPos += tChar->xSize;
				if(xPos-x>(mx- 16))
				{
// 					monofont_printText(bmpVram, fnPlot, xPos, yPos, ~0, ~0, police, textColor, monofontFlags, "...");
// 					return xPos;
				}
				if(tChar->ySize > yMax) yMax = tChar->ySize;
		}
		
		p++;
		
	}
	
	return xPos;
	
} 

u16 strchrcnt(const char *str, char c)
{
	u16 n = 0;
	const char *p = str;
	while((p = strchr(p, c))) { n++; p++; }
	return n;
}

int monofont_printTextInRect(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, u16 w, u16 h, const monofont_char *police, u16 textColor, u16 monofontFlags, const char *text)
{
	char splitBuffer[256];
	u16 xp, yp, xs;
	const char *p, *q;
	u16 nl, cl;
	u8 end = 0;
	
	cl = 0;
	nl = strchrcnt(text, '\n') + 1;
	p = q = text;
	do
	{
		p = strchr(p, '\n');
		if(p == NULL) { p = text+strlen(text); end = 1; }
			
		strncpy(splitBuffer, q, (p-q));
		splitBuffer[p-q] = '\0';
		
		xs = monofont_computeLineSize(police, splitBuffer);
		xp = x + (w / 2) - (xs / 2);
		yp = y + (h / 2) - (nl*12)/2+cl*12;
		monofont_printText(bmpVram, fnPlot, xp, yp, w, h, police, textColor, monofontFlags & ~MONOFONT_ONLYCOMPUTE, splitBuffer);
		
		
		cl++;
		p++;
		q = p;
	}
	while(!end);
	
	return 0;
}

monofont_return monofont_printTextCenter(u8 *bmpVram, monofont_fnPlot *fnPlot, u16 x, u16 y, u16 mx, u16 my, const monofont_char *police, u16 textColor, u16 monofontFlags, const char *text)
{
	u16 xC, yC,w;
	u16 xSize = monofont_computeLineSize(police, text);
	xC = x - xSize / 2;
	yC = y - 2;
	w=monofont_printText(bmpVram, fnPlot, xC, yC, mx, my, police, textColor, monofontFlags & ~MONOFONT_ONLYCOMPUTE, text);
	return (monofont_return){xC,w-xC};
}


//////////////////////// fonction plottantes ///////////////////

void plotOnTiles4(u8 *charblock, u16 mw, u16 x, u16 y, u16 color)
{
	u8 *tile;
	u8 *pixels2;
	u16 *pixels4;
	u16 padding;
	
	/* Version non-optimise : tile = charblock + (y / 8 * (ZdsVideo_screenWidth / 8) + x / 8) * 64 / 2; */
	tile = charblock + (((y >> 3) * mw + (x >> 3)) << 5);
	
	/* Version non-optimise : pixels2 = tile + (y % 8 * 8 + x % 8) / 2; */
	pixels2 = tile + ((((y & 7) << 3) + (x & 7)) >> 1);
	pixels4 = (u16*)pixels2;
	
	padding = (x & 3) << 2;
	
	*pixels4 &= ~(0xF << padding);
	*pixels4 |= color << padding;
}

void plotOnTiles8(u8 *charblock, u16 mw, u16 x, u16 y, u16 color)
{
	u8 *tile;
	u16 *pixels2;
		
	/* Version non-optimise : tile = charblock + (y / 8 * (ZdsVideo_screenWidth / 8) + x / 8) * 64 ; */
	tile = charblock + (((y >> 3) * mw + (x >> 3)) << 6);
	
	/* Version non-optimise : pixels2 = tile + (y % 8 * 8 + x % 8); */
	pixels2 = (u16*)(tile + (((y & 7) << 3) + (x & 7)));
	
	*pixels2 &= (0xFF00 >> ((x&1)<<3));
	*pixels2 |= color << ((x&1)<<3);
	
}

void plotOnBmp16(u8 *bmp, u16 mw, u16 x, u16 y, u16 color)
{
	((u16*)bmp)[x + y * (mw << 3)] = color | (1 << 15);
}



